@@ -7,6 +7,7 @@ module Rack
   class Lint
     def initialize(app)
       @app = app
+      @content_length = nil
     end
 
     # :stopdoc:
@@ -51,17 +52,18 @@ module Rack
       check_headers headers
       ## and the *body*.
       check_content_type status, headers
-      check_content_length status, headers, env
+      check_content_length status, headers
+      @head_request = env["REQUEST_METHOD"] == "HEAD"
       [status, headers, self]
     end
 
     ## == The Environment
     def check_env(env)
-      ## The environment must be an true instance of Hash (no
-      ## subclassing allowed) that includes CGI-like headers.
-      ## The application is free to modify the environment.
+      ## The environment must be an instance of Hash that includes
+      ## CGI-like headers.  The application is free to modify the
+      ## environment.
       assert("env #{env.inspect} is not a Hash, but #{env.class}") {
-        env.instance_of? Hash
+        env.kind_of? Hash
       }
 
       ##
@@ -111,7 +113,7 @@ module Rack
       ## In addition to this, the Rack environment must include these
       ## Rack-specific variables:
 
-      ## <tt>rack.version</tt>:: The Array [1,0], representing this version of Rack.
+      ## <tt>rack.version</tt>:: The Array [1,1], representing this version of Rack.
       ## <tt>rack.url_scheme</tt>:: +http+ or +https+, depending on the request URL.
       ## <tt>rack.input</tt>:: See below, the input stream.
       ## <tt>rack.errors</tt>:: See below, the error stream.
@@ -148,6 +150,35 @@ module Rack
         }
       end
 
+      ## <tt>rack.logger</tt>:: A common object interface for logging messages.
+      ##                        The object must implement:
+      if logger = env['rack.logger']
+        ##                         info(message, &block)
+        assert("logger #{logger.inspect} must respond to info") {
+          logger.respond_to?(:info)
+        }
+
+        ##                         debug(message, &block)
+        assert("logger #{logger.inspect} must respond to debug") {
+          logger.respond_to?(:debug)
+        }
+
+        ##                         warn(message, &block)
+        assert("logger #{logger.inspect} must respond to warn") {
+          logger.respond_to?(:warn)
+        }
+
+        ##                         error(message, &block)
+        assert("logger #{logger.inspect} must respond to error") {
+          logger.respond_to?(:error)
+        }
+
+        ##                         fatal(message, &block)
+        assert("logger #{logger.inspect} must respond to fatal") {
+          logger.respond_to?(:fatal)
+        }
+      end
+
       ## The server or the application can store their own data in the
       ## environment, too.  The keys must contain at least one dot,
       ## and should be prefixed uniquely.  The prefix <tt>rack.</tt>
@@ -175,7 +206,7 @@ module Rack
       env.each { |key, value|
         next  if key.include? "."   # Skip extensions
         assert("env variable #{key} has non-string value #{value.inspect}") {
-          value.instance_of? String
+          value.kind_of? String
         }
       }
 
@@ -184,7 +215,7 @@ module Rack
 
       ## * <tt>rack.version</tt> must be an array of Integers.
       assert("rack.version must be an Array, was #{env["rack.version"].class}") {
-        env["rack.version"].instance_of? Array
+        env["rack.version"].kind_of? Array
       }
       ## * <tt>rack.url_scheme</tt> must either be +http+ or +https+.
       assert("rack.url_scheme unknown: #{env["rack.url_scheme"].inspect}") {
@@ -233,8 +264,17 @@ module Rack
     ## === The Input Stream
     ##
     ## The input stream is an IO-like object which contains the raw HTTP
-    ## POST data. If it is a file then it must be opened in binary mode.
+    ## POST data.
     def check_input(input)
+      ## When applicable, its external encoding must be "ASCII-8BIT" and it
+      ## must be opened in binary mode, for Ruby 1.9 compatibility.
+      assert("rack.input #{input} does not have ASCII-8BIT as its external encoding") {
+        input.external_encoding.name == "ASCII-8BIT"
+      } if input.respond_to?(:external_encoding)
+      assert("rack.input #{input} is not opened in binary mode") {
+        input.binmode?
+      } if input.respond_to?(:binmode?)
+
       ## The input stream must respond to +gets+, +each+, +read+ and +rewind+.
       [:gets, :each, :read, :rewind].each { |method|
         assert("rack.input #{input} does not respond to ##{method}") {
@@ -250,17 +290,13 @@ module Rack
         @input = input
       end
 
-      def size
-        @input.size
-      end
-
       ## * +gets+ must be called without arguments and return a string,
       ##   or +nil+ on EOF.
       def gets(*args)
         assert("rack.input#gets called with arguments") { args.size == 0 }
         v = @input.gets
         assert("rack.input#gets didn't return a String") {
-          v.nil? or v.instance_of? String
+          v.nil? or v.kind_of? String
         }
         v
       end
@@ -291,18 +327,18 @@ module Rack
             args[1].kind_of?(String)
           }
         end
-        
+
         v = @input.read(*args)
-        
+
         assert("rack.input#read didn't return nil or a String") {
-          v.nil? or v.instance_of? String
+          v.nil? or v.kind_of? String
         }
         if args[0].nil?
           assert("rack.input#read(nil) returned nil on EOF") {
             !v.nil?
           }
         end
-        
+
         v
       end
 
@@ -311,12 +347,12 @@ module Rack
         assert("rack.input#each called with arguments") { args.size == 0 }
         @input.each { |line|
           assert("rack.input#each didn't yield a String") {
-            line.instance_of? String
+            line.kind_of? String
           }
           yield line
         }
       end
-      
+
       ## * +rewind+ must be called without arguments. It rewinds the input
       ##   stream back to the beginning. It must not raise Errno::ESPIPE:
       ##   that is, it may not be a pipe or a socket. Therefore, handler
@@ -364,7 +400,7 @@ module Rack
 
       ## * +write+ must be called with a single argument that is a String.
       def write(str)
-        assert("rack.errors#write not called with a String") { str.instance_of? String }
+        assert("rack.errors#write not called with a String") { str.kind_of? String }
         @error.write str
       end
 
@@ -398,7 +434,7 @@ module Rack
       header.each { |key, value|
         ## The header keys must be Strings.
         assert("header key must be a string, was #{key.class}") {
-          key.instance_of? String
+          key.kind_of? String
         }
         ## The header must not contain a +Status+ key,
         assert("header must not contain Status") { key.downcase != "status" }
@@ -443,7 +479,7 @@ module Rack
     end
 
     ## === The Content-Length
-    def check_content_length(status, headers, env)
+    def check_content_length(status, headers)
       headers.each { |key, value|
         if key.downcase == 'content-length'
           ## There must not be a <tt>Content-Length</tt> header when the
@@ -451,49 +487,43 @@ module Rack
           assert("Content-Length header found in #{status} response, not allowed") {
             not Rack::Utils::STATUS_WITH_NO_ENTITY_BODY.include? status.to_i
           }
-
-          bytes = 0
-          string_body = true
-
-          if @body.respond_to?(:to_ary)
-            @body.each { |part|
-              unless part.kind_of?(String)
-                string_body = false
-                break
-              end
-
-              bytes += Rack::Utils.bytesize(part)
-            }
-
-            if env["REQUEST_METHOD"] == "HEAD"
-              assert("Response body was given for HEAD request, but should be empty") {
-                bytes == 0
-              }
-            else
-              if string_body
-                assert("Content-Length header was #{value}, but should be #{bytes}") {
-                  value == bytes.to_s
-                }
-              end
-            end
-          end
-
-          return
+          @content_length = value
         end
       }
     end
 
+    def verify_content_length(bytes)
+      if @head_request
+        assert("Response body was given for HEAD request, but should be empty") {
+          bytes == 0
+        }
+      elsif @content_length
+        assert("Content-Length header was #{@content_length}, but should be #{bytes}") {
+          @content_length == bytes.to_s
+        }
+      end
+    end
+
     ## === The Body
     def each
       @closed = false
+      bytes = 0
+
       ## The Body must respond to +each+
+      assert("Response body must respond to each") do
+        @body.respond_to?(:each)
+      end
+
       @body.each { |part|
         ## and must only yield String values.
         assert("Body yielded non-string value #{part.inspect}") {
-          part.instance_of? String
+          part.kind_of? String
         }
+        bytes += Rack::Utils.bytesize(part)
         yield part
       }
+      verify_content_length(bytes)
+
       ##
       ## The Body itself should not be an instance of String, as this will
       ## break in Ruby 1.9.
