@@ -12,7 +12,11 @@ module Rack
   # first, since they are most specific.
 
   class URLMap
-    def initialize(map)
+    def initialize(map = {})
+      remap(map)
+    end
+
+    def remap(map)
       @mapping = map.map { |location, app|
         if location =~ %r{\Ahttps?://(.*?)(/.*)}
           host, location = $1, $2
@@ -24,24 +28,27 @@ module Rack
           raise ArgumentError, "paths need to start with /"
         end
         location = location.chomp('/')
+        match = Regexp.new("^#{Regexp.quote(location).gsub('/', '/+')}(.*)", nil, 'n')
 
-        [host, location, app]
-      }.sort_by { |(h, l, a)| [-l.size, h.to_s.size] }  # Longest path first
+        [host, location, match, app]
+      }.sort_by { |(h, l, m, a)| [h ? -h.size : (-1.0 / 0.0), -l.size] }  # Longest path first
     end
 
     def call(env)
-      path = env["PATH_INFO"].to_s.squeeze("/")
+      path = env["PATH_INFO"]
+      script_name = env['SCRIPT_NAME']
       hHost, sName, sPort = env.values_at('HTTP_HOST','SERVER_NAME','SERVER_PORT')
-      @mapping.each { |host, location, app|
+      @mapping.each { |host, location, match, app|
         next unless (hHost == host || sName == host \
           || (host.nil? && (hHost == sName || hHost == sName+':'+sPort)))
-        next unless location == path[0, location.size]
-        next unless path[location.size] == nil || path[location.size] == ?/
-        env["SCRIPT_NAME"] += location
-        env["PATH_INFO"]    = path[location.size..-1]
+        next unless path.to_s =~ match && rest = $1
+        next unless rest.empty? || rest[0] == ?/
+        env.merge!('SCRIPT_NAME' => (script_name + location), 'PATH_INFO' => rest)
         return app.call(env)
       }
-      [404, {"Content-Type" => "text/plain"}, ["Not Found: #{path}"]]
+      [404, {"Content-Type" => "text/plain", "X-Cascade" => "pass"}, ["Not Found: #{path}"]]
+    ensure
+      env.merge! 'PATH_INFO' => path, 'SCRIPT_NAME' => script_name
     end
   end
 end
