@@ -2,10 +2,15 @@ require 'yaml'
 require 'net/http'
 
 class TestRequest
+  NOSERIALIZE = [Method, Proc]
+
   def call(env)
     status = env["QUERY_STRING"] =~ /secret/ ? 403 : 200
     env["test.postdata"] = env["rack.input"].read
-    body = env.to_yaml
+    minienv = env.dup
+    # This may in the future want to replace with a dummy value instead.
+    minienv.delete_if { |k,v| NOSERIALIZE.any? { |c| v.kind_of?(c) } }
+    body = minienv.to_yaml
     size = body.respond_to?(:bytesize) ? body.bytesize : body.size
     [status, {"Content-Type" => "text/yaml", "Content-Length" => size.to_s}, [body]]
   end
@@ -13,6 +18,17 @@ class TestRequest
   module Helpers
     attr_reader :status, :response
 
+    ROOT = File.expand_path(File.dirname(__FILE__) + "/..")
+    ENV["RUBYOPT"] = "-I#{ROOT}/lib -rubygems"
+
+    def root
+      ROOT
+    end
+
+    def rackup
+      "#{ROOT}/bin/rackup"
+    end
+
     def GET(path, header={})
       Net::HTTP.start(@host, @port) { |http|
         user = header.delete(:user)
@@ -22,7 +38,11 @@ class TestRequest
         get.basic_auth user, passwd  if user && passwd
         http.request(get) { |response|
           @status = response.code.to_i
-          @response = YAML.load(response.body)
+          begin
+            @response = YAML.load(response.body)
+          rescue TypeError, ArgumentError
+            @response = nil
+          end
         }
       }
     end
